library angular2.src.core.compiler.pipeline.directive_parser;

import 'package:angular2/src/facade/lang.dart'
    show isPresent, isBlank, BaseException, assertionsEnabled, RegExpWrapper;
import 'package:angular2/src/facade/collection.dart'
    show List, MapWrapper, StringMapWrapper;
import 'package:angular2/src/dom/dom_adapter.dart' show DOM;
import '../selector.dart' show SelectorMatcher;
import '../selector.dart' show CssSelector;
import '../directive_metadata.dart' show DirectiveMetadata;
import '../../annotations/annotations.dart' show Component, Viewport;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
import './element_binder_builder.dart' show isSpecialProperty;
import './util.dart' show dashCaseToCamelCase, camelCaseToDashCase;

var PROPERTY_BINDING_REGEXP = RegExpWrapper.create('^ *([^\\s\\|]+)');
class DirectiveParser extends CompileStep {
  SelectorMatcher _selectorMatcher;
  DirectiveParser(List<DirectiveMetadata> directives) : super() {
    var selector;
    this._selectorMatcher = new SelectorMatcher();
    for (var i = 0; i < directives.length; i++) {
      var directiveMetadata = directives[i];
      selector = CssSelector.parse(directiveMetadata.annotation.selector);
      this._selectorMatcher.addSelectable(selector, directiveMetadata);
    }
  }
  process(
      CompileElement parent, CompileElement current, CompileControl control) {
    var attrs = current.attrs();
    var classList = current.classList();
    var cssSelector = new CssSelector();
    cssSelector.setElement(DOM.nodeName(current.element));
    for (var i = 0; i < classList.length; i++) {
      cssSelector.addClassName(classList[i]);
    }
    MapWrapper.forEach(attrs, (attrValue, attrName) {
      cssSelector.addAttribute(attrName, attrValue);
    });
    var isTemplateElement = DOM.isTemplateElement(current.element);
    var matchedProperties;
    this._selectorMatcher.match(cssSelector, (selector, directive) {
      matchedProperties =
          updateMatchedProperties(matchedProperties, selector, directive);
      checkDirectiveValidity(directive, current, isTemplateElement);
      current.addDirective(directive);
    });
    checkMissingDirectives(current, matchedProperties, isTemplateElement);
  }
}
updateMatchedProperties(matchedProperties, selector, directive) {
  if (assertionsEnabled()) {
    var attrs = selector.attrs;
    if (!isPresent(matchedProperties)) {
      matchedProperties = StringMapWrapper.create();
    }
    if (isPresent(attrs)) {
      for (var idx = 0; idx < attrs.length; idx += 2) {
        StringMapWrapper.set(
            matchedProperties, dashCaseToCamelCase(attrs[idx]), true);
      }
    }
    if (isPresent(directive.annotation) &&
        isPresent(directive.annotation.bind)) {
      var bindMap = directive.annotation.bind;
      StringMapWrapper.forEach(bindMap, (value, key) {
        var bindProp = RegExpWrapper.firstMatch(PROPERTY_BINDING_REGEXP, value);
        if (isPresent(bindProp) && isPresent(bindProp[1])) {
          StringMapWrapper.set(
              matchedProperties, dashCaseToCamelCase(bindProp[1]), true);
        }
      });
    }
  }
  return matchedProperties;
}
checkDirectiveValidity(directive, current, isTemplateElement) {
  if (directive.annotation is Viewport) {
    if (!isTemplateElement) {
      throw new BaseException(
          '''Viewport directives need to be placed on <template> elements or elements ''' +
              '''with template attribute - check ${current.elementDescription}''');
    } else if (isPresent(current.viewportDirective)) {
      throw new BaseException(
          '''Only one viewport directive can be used per element - check ${current.elementDescription}''');
    }
  } else if (isTemplateElement) {
    throw new BaseException(
        '''Only template directives are allowed on template elements - check ${current.elementDescription}''');
  } else if ((directive.annotation is Component) &&
      isPresent(current.componentDirective)) {
    throw new BaseException(
        '''Multiple component directives not allowed on the same element - check ${current.elementDescription}''');
  }
}
checkMissingDirectives(current, matchedProperties, isTemplateElement) {
  if (assertionsEnabled()) {
    var ppBindings = current.propertyBindings;
    if (isPresent(ppBindings)) {
      MapWrapper.forEach(ppBindings, (expression, prop) {
        if (!DOM.hasProperty(current.element, prop) &&
            !isSpecialProperty(prop)) {
          if (!isPresent(matchedProperties) ||
              !isPresent(StringMapWrapper.get(matchedProperties, prop))) {
            throw new BaseException(
                '''Missing directive to handle \'${camelCaseToDashCase(prop)}\' in ${current.elementDescription}''');
          }
        }
      });
    }
    if (isTemplateElement &&
        !current.isViewRoot &&
        !isPresent(current.viewportDirective)) {
      throw new BaseException(
          '''Missing directive to handle: ${current.elementDescription}''');
    }
  }
}
